/*
 Author : MEZIERE Fabien
 Project : SRO2002
 Website : https://www.instructables.com/member/-SRO2-/
 Description : This is the source code of my CheminElectrique project, for more information or any suggestion go to the website ;)

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. 1
*/

#include "E:\Projets electronique\CheminElectrique\Programmation\CheminElectrique.h"

// Pin declaration
// Set input
const char ContactWire = PIN_B0;
const char BuzzerEnabled = PIN_B6;
const char VibratorEnabled = PIN_B7;
const char HighSensitivitySelected = PIN_B2;
const char LowSensitivitySelected = PIN_B1;
const char StartBuzzer = PIN_A7;
const char StartVibrator = PIN_A6;
// Set output
const char GreenLedLightUp = PIN_A0;
const char YellowOneLedLightUp = PIN_A1;
const char YellowTwoLedLightUp = PIN_A2;
const char RedLedLightUp = PIN_A3;
const char LedStartLine = PIN_B4;
const char LedFinishLine = PIN_B5;

// Constant declaration
const long LOW_SENSITIVITY_VALUE = 500;
const long MEDIUM_SENSITIVITY_VALUE = 200;
const long HIGH_SENSITIVITY_VALUE = 50;

// Variable declaration
short flagStateGreenLed=0;
short flagStateYellowOneLed=0;
short flagStateRedLed=0;
short flagStateYellowTwoLed=0;
short ledLineMultiplex=0;
int countMultiplex=0;

// Function declaration
long GetSensitivityValue();
int GetDeviceConfiguration();
void TurnOffLed();



// Overflow RTCC 2ms for multiplex display
// LED Green, YellowOne, YellowTwo, Red are power on 2/3 of period time
// LedStartLine and LedFinishLine lights up once every 2ms
#int_RTCC
RTCC_isr() 
{
	if (ledLineMultiplex==0)
	{
		output_low(LedFinishLine);
		delay_us(10);
		output_high(LedStartLine);
		ledLineMultiplex=1;
	}
	else
	{
		output_high(LedFinishLine);
		delay_us(10);
		output_low(LedStartLine);
		ledLineMultiplex=0;
	}
	
	countMultiplex++;
	if (countMultiplex==1)
	{
		output_low(RedLedLightUp);
		output_low(YellowTwoLedLightUp);
		output_low(YellowOneLedLightUp);
		output_low(GreenLedLightUp);
	}
	if (countMultiplex==2 || countMultiplex==3)
	{
		if (flagStateGreenLed==1)
		{
			output_high(GreenLedLightUp);
		}
		if (flagStateYellowOneLed==1)
		{
			output_high(YellowOneLedLightUp);
		}
		if (flagStateYellowTwoLed==1)
		{
			output_high(YellowTwoLedLightUp);
		}
		if (flagStateRedLed==1)
		{
			output_high(RedLedLightUp);
		}
		countMultiplex=0;
	}
}

void main()
{

   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_8);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   enable_interrupts(INT_RTCC);
   enable_interrupts(GLOBAL);

	flagStateGreenLed=1;

	while(1)
	{
		if(input(ContactWire) == 1)						// If there is a contact between wire and stick
		{
			delay_ms(GetSensitivityValue()/4);			// Wait sensitivity value/4 (time in ms)
			if(input(ContactWire) == 1)					// If there is always a contact
			{
				flagStateYellowOneLed=1;				// Light on the first yellow led
				delay_ms(GetSensitivityValue()/4);		// Wait sensitivity value/4 (time in ms)
				if(input(ContactWire) == 1)				// If there is always a contact
				{
					flagStateYellowTwoLed=1;			// Light on the second yellow led
					if (GetDeviceConfiguration() == 3)	// Detect if vibrator option is enabled
					{
						output_high(StartVibrator);		// Start the vibrator
					}	
					delay_ms(GetSensitivityValue()/4);	// Wait sensitivity value/4 (time in ms)
					if(input(ContactWire) == 1)			// If there is always a contact
					{
						flagStateRedLed=1;				// Light on the red led
						if (GetDeviceConfiguration() == 2)	// Detect if buzzer option is enabled
						{ 
							output_high(StartBuzzer);	// Start the buzzer
						}
						else if (GetDeviceConfiguration() == 3) // Detect if buzzer and vibrator option is enabled
						{
							output_high(StartVibrator);	// Start the vibrator
							output_high(StartBuzzer);	// Start the buzzer
						}
	
						do								// Wait until there is no contact
						{
							delay_ms(50);
						}while(input(ContactWire) == 1);
						output_low(StartVibrator);		// Stop the vibrator
						output_low(StartBuzzer);		// Stop the buzzer
						TurnOffLed();
					}else{
						output_low(StartVibrator);		// Stop the vibrator
						output_low(StartBuzzer);		// Stop the buzzer
						TurnOffLed();
					}
				}else{
					output_low(StartVibrator);		// Stop the vibrator
					output_low(StartBuzzer);		// Stop the buzzer
					TurnOffLed();
				}
			}		
		}else{
			output_low(StartVibrator);		// Stop the vibrator
			output_low(StartBuzzer);		// Stop the buzzer
			TurnOffLed();
		}				
	}
}

// ---------------------- GetSensitivityValue() ---------------------- 
// Return 1 if Low sensitivity is set
// Return 2 if Medium sensitivity is set
// Return 3 if High sensitivity is set
long GetSensitivityValue()
{
	if(input(LowSensitivitySelected) == 1 && input(HighSensitivitySelected) == 0){
		return LOW_SENSITIVITY_VALUE;
	}else if (input(LowSensitivitySelected) == 0 && input(HighSensitivitySelected) == 0){
		return MEDIUM_SENSITIVITY_VALUE;
	}else{
		return HIGH_SENSITIVITY_VALUE;
	}
}

// ---------------------- GetDeviceConfiguration() ---------------------- 
// Return 1 if Buzzer and Vibrator are not enabled
// Return 2 if there is only Buzzer enabled
// Return 3 if Buzzer and Vibrator are enabled
int GetDeviceConfiguration()
{
	if(input(BuzzerEnabled) == 0 && input(VibratorEnabled) == 1){
		return 1;
	}else if(input(BuzzerEnabled) == 0 && input(VibratorEnabled) == 0){
		return 2;
	}else{
		return 3;
	}
}

// ---------------------- TurnOffLed() ---------------------- 
// Set LED flags to 0 to turn off the LEDs
void TurnOffLed()
{	
	flagStateYellowOneLed=0;
	flagStateYellowTwoLed=0;
	flagStateRedLed=0;
}